%% Empirical 
clear
clc
load State_Capacity_Network_Work

%% Set up
Gp = Adj; %graph
column = 1; %choose between 1,2,3,4.
dw=1; % downweighting

%Outcomes (from Acemoglu et al., we focus on the first)
semployees = log(1+S(:,1));  %log(number of employees+1)
sagencies = log(1+S(:,6)+S(:,7)+S(:,8)+S(:,11)+S(:,12)+S(:,13)+S(:,14)+S(:,15)+S(:,16)+S(:,18)+S(:,19)); %variables of state presence

%Historical Variables (Refer to Acemoglu et al. library)
Ci = [C(:,1) C(:,3) H(:,8)./1000 R R.^2 C(:,3).^2];

%Initial Definitions
LandControls = G(:,1:5);

% Column 1
if column==1
X1 = [Dc DD LandControls Ci];
X2 = [];

%Multiply by 0.0329 avg nij to compare with Acemoglu et al.

%Specification 2
elseif column==2
X1 = [Dc DD LandControls Xs(:,1) Ci];
X2=[];

%Specification 3
elseif column==3
    X1 = [Dc DD LandControls Xs(:,1) L(:,1:7) Ci];
    X2 = [];

%Speficiation 4: 
elseif column==4
    X1 = [Dc DD LandControls L(:,1:7) G(:,6:8) Ci];
    X2 = [];
end

%% Cleaning up the data by removing missing values
a1 = isnan(X1);
a2 = isnan(X2);
a3 = isnan(semployees);

%Which observations to keep.
[row, col] = find(a1);
[row2, col2] = find(a2);
[row3, col3]=find(a3);
row = [row; row2; row3];
row = unique(row);

%Cleaned Data.
X1(row,:)=[];
%X2(row,:)=[];
semployees(row,:)=[];
sagencies(row,:)=[];
Gp(row,:)=[];
Gp(:,row)=[];
Ci(row,:)=[];

%Setting up outcomes
d = size(X1,2)+ size(X2,2); %dimension of theta

%Outcome of interest.
Y_ = semployees;%outcome
n = size(X1,1);
%% Generation of what is needed outside the loop

%parameters: setting up the relevant parameters for Confidence
%Sets of gamma/rho
a_vec=ones(d,1);

C3mean = mean(C(:,3));
Rmean = mean(R);

%Note that the substitutions below reference the position of the variables
%C1, C3 and R in the X1 chosen above.  If one wants to change the order
%of variables X1, please make the appropriatement adjustments below.
a_vecC1 = zeros(d,1);
a_vecC1(end-5) = 1;

a_vecC3 = zeros(d,1);
a_vecC3(end-4,1)=1;
a_vecC3(end,1)=2*C3mean;
a_vecR = zeros(d,1);
a_vecR(end-2)=1; %I want to compute CI for variable R
%It is of quadratic form, so we must get the estimates
%for linear part, and add 2*meanR
a_vecR(end-1)=2*Rmean;
a_vecDC=zeros(d,1);
a_vecDC(1)=1;

%Parameters (Tests)
alpha = 0.05;
kappa = alpha/2;
stepsize=0.01;
ub=0.75; %Upper bound of parameter space for beta
lb=-0.75; %Lower bound for parameter space for beta
beta_1_grid=lb:stepsize:ub; %set of alternatives
beta_1_grid = beta_1_grid';
beta_1_grid_s=size(beta_1_grid,1);

%Generation of Graph characteristics
graphdata=graph_generate_fosEmpM95(n,Gp);
Np{1,1}=graphdata{2};
nP{1,1}=graphdata{3};
Gpb_temp=graphdata{4};
Gpb{1,1}=sparse(Gpb_temp);
Npb{1,1}=graphdata{5};
nPb{1,1}=graphdata{6};
Gp2_temp=graphdata{7};
Gp2{1,1}=sparse(Gp2_temp);
Np2{1,1}=graphdata{8};
nP2{1,1}=graphdata{9};
Np4_tilde{1,1}=graphdata{10};
nP4_tilde{1,1}=graphdata{11};
Np4{1,1}=graphdata{12};
nP4{1,1}=graphdata{13};
Gp2b{1,1}=graphdata{14};
     
%Storage of Test Statistics (convenience to make it similar to Monte Carlo codes
nn = 1;
ll = 1;
lam_list = [];

%Generating c_ij     
cij_matrices=cij_gen_2019(n,nP{1,1},Np{1,1}); 

%Put graph data in strucutre
    graph_=struct('Gp',Gp,'Np',Np,'nP',nP,'Gp2',Gp2,'Np2',Np2,'nP2',nP2,'Gpb',Gpb,'Npb',Npb,'nPb',...
        nPb,'Np4_tilde',Np4_tilde,'nP4_tilde',nP4_tilde,'Np4',Np4,'nP4',nP4,'Gp2b',Gp2b,...
        'lam',lam_list,'cij_matrices',cij_matrices);
   
%% Pre-allocate variables
d1=size(X1,2);
d2 = size(X2,2);
M = size([Ci,X1],2); %dimensions of our instruments. We won't use exactly these,
%but transformations of the same dimensions.
Z = cell(beta_1_grid_s,1);
 ln='n';
   zer_n_by_n=zeros(n,n); lzer_n_by_n='zer_n_by_n';
   zer_1_by_n=zeros(1,n); lzer_1_by_n='zer_1_by_n';
   zer_n_by_1=zeros(n,1); lzer_n_by_1='zer_n_by_1';
   zer_n_by_d1=zeros(n,d1); lzer_n_by_d1='zer_n_by_d1';
   zer_n_by_d2=zeros(n,d2); lzer_n_by_d2='zer_n_by_d2';
   zer_1_by_d1=zeros(1,d1); lzer_1_by_d1='zer_1_by_d1';
   zer_1_by_d2=zeros(1,d2); lzer_1_by_d2='zer_1_by_d2';
   z_=struct(lzer_1_by_d1,zer_1_by_d1,lzer_1_by_d2,zer_1_by_d2,lzer_n_by_n,zer_n_by_n,...
   lzer_1_by_n,zer_1_by_n,lzer_n_by_1,zer_n_by_1,lzer_n_by_d1,zer_n_by_d1,lzer_n_by_d2,zer_n_by_d2);

%Other storage for tests.
zeros_mm=zeros(M,M); lzeros_mm='zeros_mm';
om=ones(1,M);        lom='om';
eyem=eye(M);         leyem='eyem';
lcritval_rho='critval_rho';
zero=zeros(n,1);    lzero='zero';
lrho='rho';
rho = zeros(d1+d2,1);
ld1='d1';
ld2='d2';
lB = 'B';
la_vec='a_vec';
la_vecC1 = 'a_vecC1';
la_vecC3 = 'a_vecC3';
la_vecR = 'a_vecR';
la_vecDC = 'a_vecDC';
critval_beta=chi2inv(1-alpha,M-(d1+d2)); 
critval_beta_div2=chi2inv(1-alpha/2,M-(d1+d2)); 
critval_rho=norminv(1-(alpha-kappa)/2,0,1);
critval_half_alpha=chi2inv(1-kappa,M-(d1+d2));
one=ones(1,n); lone='one';
sqrtn=sqrt(n); lsqrtn='sqrtn';  
lalpha = 'alpha';
lkappa = 'kappa';
ar_=struct(lzeros_mm,zeros_mm,lom,om,leyem,eyem,ln,n,lone,one,lzero,zero,lsqrtn,sqrtn,la_vec,a_vec,la_vecC1, a_vecC1, la_vecC3,a_vecC3,la_vecR, a_vecR, la_vecDC, a_vecDC, lcritval_rho,critval_rho,lalpha,alpha,lkappa,kappa,lrho,rho);
arom=ones(1,M);
one_n = ones(size(X1,1),1);

%Storage of Test Statistics
stat_out = cell(beta_1_grid_s,1);
beta_conf = zeros(beta_1_grid_s,1);
beta_conf_kappa = zeros(beta_1_grid_s,1);

rhohat = cell(beta_1_grid_s,1);
Chatrho = cell(beta_1_grid_s,1);
ChatrhoC1 = cell(beta_1_grid_s,1);
ChatrhoC3 = cell(beta_1_grid_s,1);
ChatrhoR = cell(beta_1_grid_s,1);
ChatrhoDC = cell(beta_1_grid_s,1);
IU_beta = cell(beta_1_grid_s,1);

%% Run

parfor k = 1:beta_1_grid_s

%W1 matrix
out1=W_gen(n,nn,graph_,z_,ll,beta_1_grid(k));
W_fos_mat_1=out1{1};
W_fos_bar_mat_1=out1{2};

lambda_inst=lambda_gen_inst(n,nn,ll,graph_,z_,beta_1_grid(k));
sum_np_1 =(lambda_inst.*graph_(nn,ll).Gp);
                    
q_e_ij_s=q_e_ij_gen_fos(n,nn,ll,graph_,z_,beta_1_grid(k),W_fos_bar_mat_1);

%% Generating Z
Z_beta=W_fos_mat_1*X1;

%Instruments
phi_beta=[Gp*Ci,sum_np_1*X1]; %Acemoglu et al. instruments, and our own.
weight=sqrt(cell2mat(nP)+1)*arom;
if dw==1
phi_beta = phi_beta./weight;
end
sphiphi_inv=inv(phi_beta'*phi_beta/n);

%Final instruments are rescaled by magnitude of phi (our variables can have very different supports)
phi_tilde=phi_beta*sqrtm(sphiphi_inv);

%Computing Statistics (and storing them)
stat_out{k}= stats_asymptotic_fos_empM95(nn,ll,graph_,Y_,phi_tilde,Z_beta,q_e_ij_s,ar_);
beta_conf(k) = (cell2mat(stat_out{k}(1))<=critval_beta);
beta_conf_kappa(k) = (cell2mat(stat_out{k}(1))<=critval_half_alpha);

rhohat{k} = stat_out{k}(2);
Chatrho{k} = stat_out{k}(3);
ChatrhoC1{k} = stat_out{k}(4);
ChatrhoC3{k} = stat_out{k}(5);
ChatrhoR{k} = stat_out{k}(6);
ChatrhoDC{k} = stat_out{k}(7);
IU_beta{k} = stat_out{k}(8); %not used in the paper.

end

%% CI and Results for the Paper
%Create CrhoC1,CrhoC3, CrhoR which are the unions of the individual ones
%across the betas in the CS.

%Beta CI
mn = min(find(beta_conf==1)); %min index of beta_grid (beta CI, under level alpha)
mx = max(find(beta_conf==1)); %max index

%C_rho CI
mn_c = min(find(beta_conf_kappa==1)); %min index of beta_grid under level alpha/2 (for a'rho CI)
mx_c = max(find(beta_conf_kappa==1)); %max index

ChatC1 = zeros(max(size(ChatrhoC1)),2);
ChatC3 = zeros(max(size(ChatrhoC3)),2);
ChatR = zeros(max(size(ChatrhoR)),2);
ChatDC = zeros(max(size(ChatrhoDC)),2);

for k = mn_c:mx_c
ChatC1(k,:) = cell2mat(ChatrhoC1{k}); 
ChatC3(k,:) = cell2mat(ChatrhoC3{k}); 
ChatR(k,:) = cell2mat(ChatrhoR{k}); 
ChatDC(k,:) = cell2mat(ChatrhoDC{k}); 
end

disp('Appendix Table - Empirical Results for FOS')

disp(['Column' num2str(column)])

if column<=2
disp(['The beta CS is ' num2str(beta_1_grid(mn)) '  ' num2str(beta_1_grid(mx))])
else
disp(['The minimum and maximum values in beta CS are ' num2str(beta_1_grid(mn)) '  ' num2str(beta_1_grid(mx))])
disp('Check variable beta_conf for details')
end

disp(['The dy_i/d(colonial_state_officials)_i is ' num2str(min(ChatC1(mn_c:mx_c,1))) '  ' num2str(max(ChatC1(mn_c:mx_c,2)))])
disp(['The dy_i/d(colonial_state_agencies)_i is ' num2str(min(ChatC3(mn_c:mx_c,1))) '  ' num2str(max(ChatC3(mn_c:mx_c,2)))])
disp(['The dy_i/d(distance_royal_roads)_i is ' num2str(min(ChatR(mn_c:mx_c,1))) '  ' num2str(max(ChatR(mn_c:mx_c,2)))])

%% Model Selection: Simple Type (done in the other code) vs. First Order Sophisticated

T_FS = zeros(beta_1_grid_s,1);

for k =1:beta_1_grid_s
T_FS(k) = cell2mat(stat_out{k}(1));
end

FS_selected = (min(T_FS(logical(beta_conf)))<critval_beta_div2); 

%p-value (stat follows chi2(M-d))
p_val_ModelSelection_FS = 1-chi2cdf(min(T_FS(logical(beta_conf))),M-d);

%critical value
crit_ModelSelection = chi2inv(0.975,M-d);

%Output in the Text
disp('Appendix Table - Model Selection Results for FOS Types')

disp(['Column' num2str(column)])

disp(['Model Selection for FS: FS Selected ' num2str(FS_selected)])
disp(['Model Selection for FS: Test Statistic ' num2str(min(T_FS(logical(beta_conf))))])
disp(['Model Selection for FS: p-value ' num2str(p_val_ModelSelection_FS)])