%% Empirical 
clear
clc
load State_Capacity_Network_Work

%% Graph Statistics
Gp=Adj;

avg_deg = mean(sum(Gp,1));
max_deg = max(sum(Gp,1));
min_deg = min(sum(Gp,1));

disp(['Figure - Network in Empirical Application'])

disp(['The average degree is ' num2str(avg_deg)])
disp(['The maximum degree is ' num2str(max_deg)])
disp(['The minimum degree is ' num2str(min_deg)])
 
%Degree distribution
hist(sum(Gp,1),1:20)
set(gca,'linewidth',2,'fontsize',14,'fontweight','bold')
%title('Degree Distribution of Gp (Adjacent Colombian Municipalities)','FontSize',16,'Fontweight','bold')
xlabel('Degree','FontSize',16,'Fontweight','bold');
ylabel('Frequency','FontSize',16,'Fontweight','bold');


%% Set up
column = 1; %choose between 1,2,3,4.
dw=1; % downweighting

%Outcomes (from Acemoglu et al., we focus on the first one)
semployees = log(1+S(:,1)); %number of employees
sagencies = log(1+S(:,6)+S(:,7)+S(:,8)+S(:,11)+S(:,12)+S(:,13)+S(:,14)+S(:,15)+S(:,16)+S(:,18)+S(:,19)); %number of state

%Historical Variables (Refer to Acemoglu et al. library)
Ci = [C(:,1) C(:,3) H(:,8)./1000 R R.^2 C(:,3).^2];

%Initial Definitions
LandControls = G(:,1:5);

% Specification 1
if column==1
X1 = [Dc DD LandControls Ci];
X2 = [];

%Specification 2
elseif column==2
X1 = [Dc DD LandControls Xs(:,1) Ci];
X2=[];

%Specification 3
elseif column==3
    X1 = [Dc DD LandControls Xs(:,1) L(:,1:7) Ci];
    X2 = [];

%Speficiation 4: 
elseif column==4
    X1 = [Dc DD LandControls L(:,1:7) G(:,6:8) Ci];
    X2 = [];
end

%% Cleaning up the data by removing missing values
a1 = isnan(X1);
a2 = isnan(X2);
a3 = isnan(semployees);

%Which observations to keep.
[row, col] = find(a1);
[row2, col2] = find(a2);
[row3, col3]=find(a3);
row = [row; row2; row3];
row = unique(row);

%Cleaned Data.
X1(row,:)=[];
%X2(row,:)=[];
semployees(row,:)=[];
% sagencies(row,:)=[];
Gp(row,:)=[];
Gp(:,row)=[];
Ci(row,:)=[];

%Setting up outcomes
d = size(X1,2)+ size(X2,2); %dimension of theta

%Outcome of interest.
if outcome==1
Y_ = semployees;%outcome
else
Y_ = sagencies;
end
n = size(X1,1);
%% Generation of what is needed outside the loop

%parameters: setting up the relevant parameters for Confidence
%Sets of gamma/rho
a_vec=ones(d,1);

C3mean = mean(C(:,3));
Rmean = mean(R);

%Note that the substitutions below reference the position of the variables
%C1, C3 and R in the X1 chosen above.  If one wants to change the order
%of variables X1, please make the appropriatement adjustments below.
a_vecC1 = zeros(d,1);
a_vecC1(end-5) = 1;

a_vecC3 = zeros(d,1);
a_vecC3(end-4,1)=1;
a_vecC3(end,1)=2*C3mean;
a_vecR = zeros(d,1);
a_vecR(end-2)=1; %I want to compute CI for variable R
%It is of quadratic form, so we must get the estimates
%for linear part, and add 2*meanR
a_vecR(end-1)=2*Rmean;
a_vecDC=zeros(d,1);
a_vecDC(1)=1;

%Parameters (Tests)
alpha = 0.05;
kappa = alpha/2;
stepsize=0.01;
ub=0.75; %Upper bound of parameter space for beta
lb=-0.75; %Lower bound for parameter space for beta
beta_1_grid=lb:stepsize:ub; %set of alternatives
beta_1_grid = beta_1_grid';
beta_1_grid_s=size(beta_1_grid,1);

%Generation of Graph characteristics
Gpb= Gp+eye(n);
graphdata=graph_generate_empM85(n,Gp);
Np{1,1} = graphdata{2};
nP{1,1} = graphdata{3};
Gp2{1,1} = graphdata{4};
Np2{1,1} = graphdata{5};
nP2{1,1} = graphdata{6};
Gp2_tilde{1,1}=graphdata{7};
Np2_tilde{1,1}=graphdata{8};
nP2_tilde{1,1}=graphdata{9};

%Storage of Test Statistics (convenience to make it similar to Monte Carlo codes
nn = 1;
ll = 1;
lam_list = [];

%Generating c_ij     
cij_matrices=cij_gen_2019(n,nP{1,1},Np{1,1}); 
lcij_matrices='cij_matrices';
llam='lam'; lGp='Gp'; lGp2='Gp2'; lNp='Np'; lNp2='Np2'; lnP='nP'; lnP2='nP2';
lGp2_tilde='Gp2_tilde'; lNp2_tilde='Np2_tilde'; lnP2_tilde='nP2_tilde';

%Put graph data in structure
graph_=struct(lGp,Gp,lNp,Np,lnP,nP,lGp2,Gp2,lNp2,Np2,lnP2,nP2,lGp2_tilde,Gp2_tilde,lNp2_tilde,Np2_tilde,lnP2_tilde,nP2_tilde,...
    llam,lam_list,lcij_matrices,cij_matrices);

%% Pre-allocate variables
d1=size(X1,2);
d2 = size(X2,2);
M = size([Ci,X1],2); %dimensions of our instruments. We won't use exactly these,
%but transformations of the same dimensions.
Z = cell(beta_1_grid_s,1);
 ln='n';
   zer_n_by_n=zeros(n,n); lzer_n_by_n='zer_n_by_n';
   zer_1_by_n=zeros(1,n); lzer_1_by_n='zer_1_by_n';
   zer_n_by_1=zeros(n,1); lzer_n_by_1='zer_n_by_1';
   zer_n_by_d1=zeros(n,d1); lzer_n_by_d1='zer_n_by_d1';
   zer_n_by_d2=zeros(n,d2); lzer_n_by_d2='zer_n_by_d2';
   zer_1_by_d1=zeros(1,d1); lzer_1_by_d1='zer_1_by_d1';
   zer_1_by_d2=zeros(1,d2); lzer_1_by_d2='zer_1_by_d2';
   z_=struct(lzer_1_by_d1,zer_1_by_d1,lzer_1_by_d2,zer_1_by_d2,lzer_n_by_n,zer_n_by_n,...
   lzer_1_by_n,zer_1_by_n,lzer_n_by_1,zer_n_by_1,lzer_n_by_d1,zer_n_by_d1,lzer_n_by_d2,zer_n_by_d2);

%matrix for lambda_ij
LambdaMatrix = cell(beta_1_grid_s,1);
LambdaMatrix_np = cell(beta_1_grid_s,1);
LambdaMatrix_bars = cell(beta_1_grid_s,1);

%Other storage for tests.
zeros_mm=zeros(M,M); lzeros_mm='zeros_mm';
om=ones(1,M);        lom='om';
eyem=eye(M);         leyem='eyem';
lcritval_rho='critval_rho';
zero=zeros(n,1);    lzero='zero';
lrho='rho';
rho = zeros(d1+d2,1);
ld1='d1';
ld2='d2';
lB = 'B';
la_vec='a_vec';
la_vecC1 = 'a_vecC1';
la_vecC3 = 'a_vecC3';
la_vecR = 'a_vecR';
la_vecDC = 'a_vecDC';
critval_beta=chi2inv(1-alpha,M-(d1+d2)); 
critval_beta_div2=chi2inv(1-alpha/2,M-(d1+d2)); 
critval_rho=norminv(1-(alpha-kappa)/2,0,1);
critval_half_alpha=chi2inv(1-kappa,M-(d1+d2));
one=ones(1,n); lone='one';
sqrtn=sqrt(n); lsqrtn='sqrtn';  
lalpha = 'alpha';
lkappa = 'kappa';
ar_=struct(lzeros_mm,zeros_mm,lom,om,leyem,eyem,ln,n,lone,one,lzero,zero,lsqrtn,sqrtn,la_vec,a_vec,la_vecC1, a_vecC1, la_vecC3,a_vecC3,la_vecR, a_vecR, la_vecDC, a_vecDC, lcritval_rho,critval_rho,lalpha,alpha,lkappa,kappa,lrho,rho);
arom=ones(1,M);
one_n = ones(n,1);

%Storage of Test Statistics
stat_out = cell(beta_1_grid_s,1);
beta_conf = zeros(beta_1_grid_s,1);
beta_conf_kappa = zeros(beta_1_grid_s,1);

w0_ij_store = cell(beta_1_grid_s,1);
rhohat = cell(beta_1_grid_s,1);
Chatrho = cell(beta_1_grid_s,1);
ChatrhoC1 = cell(beta_1_grid_s,1);
ChatrhoC3 = cell(beta_1_grid_s,1);
ChatrhoR = cell(beta_1_grid_s,1);
ChatrhoDC = cell(beta_1_grid_s,1);
IU_beta = cell(beta_1_grid_s,1);

%% Run

parfor k = 1:beta_1_grid_s

%Lambda_ij's
out=lambda_gen_2019(n,nn,ll,graph_,z_,beta_1_grid(k)); 
LambdaMatrix{k} = out{1};
LambdaMatrix_np{k}=out{2};
LambdaMatrix_bars{k}=out{3};

%W0 matrix
w0_1 = one_n+beta_1_grid(k)^2*LambdaMatrix_bars{k}./(graph_(nn,ll).nP-beta_1_grid(k)^2*LambdaMatrix_bars{k});
w0_1(isnan(w0_1))=0; %Because empty payoff neighborhood
w0vec_1=w0_1*ones(1,d1+d2);
w0_ij_store{k} = beta_1_grid(k)*LambdaMatrix{k}.*((w0_1./nP{1,1})*(ones(1,n)));

%% Generating Z

%Generate q_e_ij, Z
q_e_ij_s = q_e_ij_gen(n,nn,ll,graph_,z_,beta_1_grid(k),LambdaMatrix{k},w0_1);
Z_beta=(X1+beta_1_grid(k)*(LambdaMatrix_np{k}.*Gp)*X1).*w0vec_1;

%Instruments
phi_beta=[Gp*Ci,LambdaMatrix_np{k}.*Gp*X1]; %Acemoglu et al. instruments, and our own.
weight=sqrt(cell2mat(nP)+1)*arom;
if dw==1
phi_beta = phi_beta./weight;
end
sphiphi_inv=inv(phi_beta'*phi_beta/n);

%Final instruments are rescaled by magnitude of phi (our variables can have very different supports)
phi_tilde=phi_beta*sqrtm(sphiphi_inv);

%Computing Statistics (and storing them)
stat_out{k}= stats_asymptotic_empM85(nn,ll,graph_,Y_,phi_tilde,Z_beta,q_e_ij_s,ar_);
beta_conf(k) = (cell2mat(stat_out{k}(1))<=critval_beta);
beta_conf_kappa(k) = (cell2mat(stat_out{k}(1))<=critval_half_alpha);

rhohat{k} = stat_out{k}(2);
Chatrho{k} = stat_out{k}(3);
ChatrhoC1{k} = stat_out{k}(4);
ChatrhoC3{k} = stat_out{k}(5);
ChatrhoR{k} = stat_out{k}(6);
ChatrhoDC{k} = stat_out{k}(7);
IU_beta{k} = stat_out{k}(8); %not used in the paper.

end

%% CI and Results for the Paper
%Create CrhoC1,CrhoC3, CrhoR which are the unions of the individual ones
%across the betas in the CS.

%Beta CI
mn = min(find(beta_conf==1)); %min index of beta_grid (beta CI)
mx = max(find(beta_conf==1)); %max index

%C_rho CI
mn_c = min(find(beta_conf_kappa==1)); %min index of beta_grid (beta CI)
mx_c = max(find(beta_conf_kappa==1)); %max index

ChatC1 = zeros(max(size(ChatrhoC1)),2);
ChatC3 = zeros(max(size(ChatrhoC3)),2);
ChatR = zeros(max(size(ChatrhoR)),2);
ChatDC = zeros(max(size(ChatrhoDC)),2);

for k = mn_c:mx_c
ChatC1(k,:) = cell2mat(ChatrhoC1{k}); 
ChatC3(k,:) = cell2mat(ChatrhoC3{k}); 
ChatR(k,:) = cell2mat(ChatrhoR{k}); 
ChatDC(k,:) = cell2mat(ChatrhoDC{k}); 
end

disp('Table - Empirical Results for Simple Types')

disp(['Column' num2str(column)])

disp(['The beta CS is ' num2str(beta_1_grid(mn)) '  ' num2str(beta_1_grid(mx))])
disp(['The dy_i/d(colonial_state_officials)_i is ' num2str(min(ChatC1(mn_c:mx_c,1))) '  ' num2str(max(ChatC1(mn_c:mx_c,2)))])
disp(['The dy_i/d(colonial_state_agencies)_i is ' num2str(min(ChatC3(mn_c:mx_c,1))) '  ' num2str(max(ChatC3(mn_c:mx_c,2)))])
disp(['The dy_i/d(distance_royal_roads)_i is ' num2str(min(ChatR(mn_c:mx_c,1))) '  ' num2str(max(ChatR(mn_c:mx_c,2)))])

%% Model Selection: Simple Type vs. First Order
%Here we only present the statistic for Simple Types.
%FOS Statistic done in the code for estimation of First Order model.

T_ST = zeros(beta_1_grid_s,1);

for k =1:beta_1_grid_s
T_ST(k) = cell2mat(stat_out{k}(1));
end

%Model Selection
ST_selected = (min(T_ST(logical(beta_conf)))<critval_beta_div2); 

%p-value (stat follows chi2(M-d))
p_val_ModelSelection_ST = 1-chi2cdf(min(T_ST(logical(beta_conf))),M-d);

%critical value
crit_ModelSelection = chi2inv(0.975,M-d);


%Output in the Text
disp('Appendix Table - Model Selection Results for Simple Types')

disp(['Column' num2str(column)])

disp(['Model Selection for ST: ST Selected ' num2str(ST_selected)])
disp(['Model Selection for ST: Test Statistic ' num2str(min(T_ST(logical(beta_conf))))])
disp(['Model Selection for ST: p-value ' num2str(p_val_ModelSelection_ST)])

%% Avg Network Externality

if column==3
extB=[];
extBmin=[];
extBmax=[];
mn = min(find(beta_conf_kappa==1)); %min index of beta_grid (beta CI level alpha/2)
mx = max(find(beta_conf_kappa==1)); %max index

for k =mn:mx
beta = beta_1_grid(k);
rhohat_capital = rhohat{k,1}{1,1}(1);
rhomin = ChatDC(k,1);
rhomax = ChatDC(k,2);

fi = 0;
fimin = 0;
fimax = 0;

for i = 1:n
    nptemp = nP{1,1}(i);
    for j_ = 1:nptemp
        j = Np{1,1}{i,1}(j_);
        fi = fi + rhohat_capital*w0_ij_store{k}(i,j);
        fimin = fimin + rhomin*w0_ij_store{k}(i,j);
        fimax = fimax + rhomax*w0_ij_store{k}(i,j);
    end
end
extB = [extB; fi/n];
extBmin = [extBmin; fimin/n];
extBmax = [extBmax; fimax/n];
end

% Plotting.
%confidence interval for beta.
betatemp = beta_1_grid(beta_conf_kappa>0);

grey = [0.7,0.7,0.7];
patch([betatemp', fliplr(betatemp')],[extBmin',fliplr(extBmax')],grey)
hold on
%confidence region, shaded grey.
plot(betatemp,extB,'linewidth',2.5,'color','k')
plot(betatemp,extBmin,'linewidth',1,'color','k')
plot(betatemp,extBmax,'linewidth',1,'color','k')

xlabel('\beta','FontSize',16,'Fontweight','bold');
axis([0.15,0.4,-0.5,7]); 
ylabel('Average Network Externality','FontSize',16,'Fontweight','bold')
set(gca,'linewidth',2,'fontsize',14,'fontweight','bold')
ax = gca;
ax.XAxisLocation = 'origin';
ax.YAxisLocation = 'origin';
%title('Network Externality from being a Department Capital','FontSize',16,'Fontweight','bold')
end

%% IU - Statistic Hypothesis Testing
IU_1_double = zeros(beta_1_grid_s,1);

for k = 1:beta_1_grid_s
IU_1_double(k) = cell2mat(IU_beta{k}); 
end

IU_1 = min(IU_1_double(logical(beta_conf_kappa)));

p_val_IU_1 = 1-chi2cdf(IU_1,1);
