
%% This file simulates the average network externality for the equilibrium model and behavioural models up to 10th order sophistication
% We consider two models of payoff graph generation: 1. ER and 2. BA.

%Last updated October 12, 2019

%% 1. Erdos Renyi random graphs
clear;
rng(17291234, 'twister')

beta_grid=[-0.9:0.1:0.9]';
M=size(beta_grid,1);
K=5;
mks=5;
MS=50;  %Simulation number
mmax=4;

ccext_A_ER=0;
ccext_BS_ER=0;
ccext_B1_ER=0;
ccext_B2_ER=0;
ccext_B3_ER=0;
ccext_B4_ER=0;

ccdx=0;
ccda=0;
ccn=0;
n=10000;

parfor ms=1:MS
    
    p=mks/n;
    graph=erdos_renyi(n,p,ms);               % the graph is undirected.
    graph=graph-diag(diag(graph));        % Diagonal terms are set to be zero
    
    A0=graph;
    cext_A_ER=[];
    cext_BS_ER=[];
    cext_B1_ER=[];
    cext_B2_ER=[];
    cext_B3_ER=[];
    cext_B4_ER=[];
    
    cdx=[];
    cda=[];
    Ak=0;
    R = eye(n);
    cn=[];
    for k=1:K;
        Ak=Ak+A0^k;
        
        A=A0((Ak(1,:) + R(1,:) > 0),(Ak(1,:) + R(1,:) > 0));
        nk=size(A,1);
        I=eye(nk);
        Ab=A+I;
        nP=zeros(nk,1);
        nPinv=zeros(nk,1);
        nPb=zeros(nk,1);
        Np = cell(nk,1);
        Npb = cell(nk,1);
        Np2b=cell(nk,1);
        
        A2 = (A+A^2)>0;    %new for fos code
        for i=1:nk
            A2(i,i)=0;
        end
        Np2 = cell(nk,1);  %new for fos code
        nP2 = zeros(nk,1); %new for fos code
        nP2b=zeros(nk,1);
        %find the neighbours
        for j = 1:nk
            Np{j}=find(A(j,:));
            Npb{j}=find(Ab(j,:)); %Np_bar neighbors
            nP(j)=size(Np{j},2);  %number of Np neighbors
            nPb(j)=size(Npb{j},2); %number of Npbar neighbors.
            if nP(j)>0;
                nPinv(j) = nP(j)^(-1);
            else
                nPinv(j) = 0;
            end
            
        end
        A1 = A.*(nPinv*ones(1,nk));
        ext_A_ER=[];
        ext_BS_ER=[];
        ext_B1_ER=[];
        ext_B2_ER=[];
        ext_B3_ER=[];
        ext_B4_ER=[];
        
        
        for m=1:M;
            
            [ms,k,nk,m]
            
            beta=beta_grid(m);
            
            
            Adiff=ones(nk,nk);
            for i=1:nk
                Adiff(i,i)=0;
            end                   
            %Average network externality for equilibrium model
            ext_A_ER = [ext_A_ER; sum(sum(inv(I - beta * A1).*Adiff,2))/nk];
            
            cij_matrix=cij_gen_2019(nk,nP,Np);
            out=lambda_gen_2019(nk,nP,Np,cij_matrix,beta);
            lambda_matrix=out{1};
            lambda_bars=out{2};           
            wii0s=ones(nk,1)+beta^2*lambda_bars./(nP-beta^2*lambda_bars);
            wij0s=beta*lambda_matrix.*((wii0s./nP)*(ones(1,nk)))  ;       
            
            W0=zeros(nk,nk);
            
            fi1=0;
            
            for i=1:nk  
                
                %Sum of externalities for simple type
                for j_=1:nP(i)
                    j=Np{i}(j_);
                    ijterm=(wij0s(i,j));
                    fi1=fi1+ijterm;
                end
                
                for j_=1:nPb(i)      %Consider each of i's payoff neighbors
                    j=Npb{i}(j_);    %Extract the actual index associated with this #'d of i'd payoff neighbord
                    W0(j,i)= (j==i)*wii0s(i)+wij0s(i,j)*(any(j==Np{i}));
                end
            end
            
            avext = externality_gen_2019(nk,nP,Np,beta,W0,A,mmax);
            ext_BS_ER = [ext_BS_ER;fi1/nk];   %Average Network Externality for Simple Types
            ext_B1_ER = [ext_B1_ER;avext(1)]; %Average Network Externality for First Order Sophistication
            ext_B2_ER = [ext_B2_ER;avext(2)]; %...2nd order
            ext_B3_ER = [ext_B3_ER;avext(3)]; %...third order
            ext_B4_ER = [ext_B4_ER;avext(4)]; %fourth order
          
        end
        
        
        dx = max(nP);
        da = mean(nP);
        
        cext_A_ER = [cext_A_ER ext_A_ER];
        cext_BS_ER = [cext_BS_ER ext_BS_ER];
        cext_B1_ER = [cext_B1_ER ext_B1_ER];
        cext_B2_ER = [cext_B2_ER ext_B2_ER];
        cext_B3_ER = [cext_B3_ER ext_B3_ER];
        cext_B4_ER = [cext_B4_ER ext_B4_ER];
    
        cdx = [cdx dx];
        cda = [cda da];
        cn = [cn nk];
        
    end
    
    ccext_A_ER = ccext_A_ER+cext_A_ER/MS;
    ccext_BS_ER = ccext_BS_ER+cext_BS_ER/MS;
    ccext_B1_ER = ccext_B1_ER+cext_B1_ER/MS;
    ccext_B2_ER = ccext_B2_ER+cext_B2_ER/MS;
    ccext_B3_ER = ccext_B3_ER+cext_B3_ER/MS;
    ccext_B4_ER = ccext_B4_ER+cext_B4_ER/MS;
    
    %Statistics for payoff graphs
    ccdx = ccdx + cdx/MS;
    ccda = ccda + cda/MS;
    ccn=ccn+cn/MS;
    
    
end

diary extval_sim_ER.txt

ccn
ccdx
ccda

ccext_A_ER
ccext_BS_ER
ccext_B1_ER
ccext_B2_ER
ccext_B3_ER
ccext_B4_ER


diary off


timen=datestr(datetime('now','Format','HH:mm:ss.SSS'));
cat1=strcat('ER-n-',num2str(n));
cat2=strcat(cat1,'-');
cat3=strcat(cat2,timen);
save(cat3)


beta_grid=[-0.9:0.1:0.9]';
figure;
yl=-2;
yu=2;
LT=2.5;


subplot(2,2,1);plot(beta_grid,ccext_BS_ER(:,3:5),'k');axis([-1 1 yl yu]);xlabel('\beta');ylabel('ANE')
set(findall(gcf,'-property','FontSize'),'FontSize',12)
set(findall(gca,'-property','LineWidth'),'LineWidth',LT)
title('Simple Type')

subplot(2,2,2);plot(beta_grid,ccext_B1_ER(:,3:5),'k');axis([-1 1 yl yu]);xlabel('\beta');ylabel('ANE')
set(findall(gcf,'-property','FontSize'),'FontSize',12)
set(findall(gca,'-property','LineWidth'),'LineWidth',LT)
title('1st Order')


subplot(2,2,3);plot(beta_grid,ccext_B4_ER(:,3:5),'k');axis([-1 1 yl yu]);xlabel('\beta');ylabel('ANE')
set(findall(gcf,'-property','FontSize'),'FontSize',12)
set(findall(gca,'-property','LineWidth'),'LineWidth',LT)
title('4th Order')


subplot(2,2,4);plot(beta_grid,ccext_A_ER(:,3:5),'k');axis([-1 1 yl yu]);xlabel('\beta');ylabel('ANE');
set(findall(gcf,'-property','FontSize'),'FontSize',12)
set(findall(gca,'-property','LineWidth'),'LineWidth',LT)
title('Equilibrium')


%% 2. Barabasi Albert graph generation.
clear;
clc

rng(17291234, 'twister')


beta_grid=[-0.9:.1:.9]';
M=size(beta_grid,1);
K=4;
mks=2;
MS=50;  %Simulation number

% Generate Graphs for Generating True Parameters along Barabasi-Albert.

lam=1;         % parameter for Erdos-Renyi network used as a seed.
S=20;          % The number of the nodes in the seed network.

ccext_A=0;
ccext_BS=0;
ccext_B1=0;
ccext_B2=0;
ccext_B3=0;
ccext_B4=0;

ccdx=0;
ccda=0;
ccn=0;
mmax=4;
n=10000;


seedgraphs=cell(MS,1);
parfor ms=1:MS
    tag=0;p=lam/(S-1);itr=0;
    while tag==0&&itr<200;
        itr=itr+1;
        seed=erdos_renyi(S,p,ms); % Construct a Erdos-Renyu graph of size S.
        
        if size((seed>0),1)>0; % Check whether the graph is empty or not.
            tag=1;
        end
    end
    seedgraphs{ms}=seed;
end

parfor ms=1:MS
    
    seed=seedgraphs{ms};
    seed=seed-diag(diag(seed)); % The diagonal terms are set to be zero.
    graph=barabasi_albert(n,mks,seed); % Generate a graph from Barabasi-Albert
    % the graph is in the form of an adjacency matrix.
    graph=graph-diag(diag(graph));        % Diagonal terms are set to be zero
    
    A0=graph;
    cext_A=[];
    cext_BS=[];
    cext_B1=[];
    cext_B2=[];
    cext_B3=[];
    cext_B4=[];
    
    
    cdx=[];
    cda=[];
    Ak=0;
    R = eye(n);
    cn=[];
    for k=1:K;        Ak=Ak+A0^k;
        A=A0((Ak(1,:) + R(1,:) > 0),(Ak(1,:) + R(1,:) > 0));
        nk=size(A,1);
        
        I=eye(nk);
        Ab=A+I;
        
        nP=zeros(nk,1);
        nPinv=zeros(nk,1);
        nPb=zeros(nk,1);
        Np = cell(nk,1);
        Npb = cell(nk,1);
        Np2b=cell(nk,1);
        
        A2 = (A+A^2)>0;    %new for fos code
        for i=1:nk
            A2(i,i)=0;
        end
        Np2 = cell(nk,1);  %new for fos code
        nP2 = zeros(nk,1); %new for fos code
        nP2b=zeros(nk,1);
        %find the neighbours
        for j = 1:nk
            Np{j}=find(A(j,:));
            Npb{j}=find(Ab(j,:)); %Np_bar neighbors
            nP(j)=size(Np{j},2);  %number of Np neighbors
            nPb(j)=size(Npb{j},2); %number of Npbar neighbors.
            if nP(j)>0;
                nPinv(j) = nP(j)^(-1);
            else
                nPinv(j) = 0;
            end
            
        end
        A1 = A.*(nPinv*ones(1,nk));
        ext_A=[];
        ext_BS=[];
        ext_B1=[];
        ext_B2=[];
        ext_B3=[];
        ext_B4=[];
        
        
        for m=1:M;
            
            
            [ms,k,nk,m]
            
            beta=beta_grid(m);
            Adiff=ones(nk,nk);
            for i=1:nk
                Adiff(i,i)=0;
            end
            %ext_A = [ext_A; nPinv'*sum(inv(I - beta * A1).*A,2)/nk];
            ext_A = [ext_A; sum(sum(inv(I - beta * A1).*Adiff,2))/nk];
            
            
            cij_matrix=cij_gen_2019(nk,nP,Np);   %Needs to be checked!
            out=lambda_gen_2019(nk,nP,Np,cij_matrix,beta); %Needs to be checked!
            lambda_matrix=out{1};
            lambda_bars=out{2};
            
            wii0s=ones(nk,1)+beta^2*lambda_bars./(nP-beta^2*lambda_bars);
            wij0s=beta*lambda_matrix.*((wii0s./nP)*(ones(1,nk))); %Be careful abotu empty neighborhood.
            
            
            W0=zeros(nk,nk);
    
            fi1=0;
            
            for i=1:nk  % NEW VERSION

                %Sum of externalities for simple type
                for j_=1:nP(i)
                    j=Np{i}(j_);
                    ijterm=(wij0s(i,j));
                    fi1=fi1+ijterm;
                end
                
                
                for j_=1:nPb(i)      %Consider each of i's payoff neighbors
                    j=Npb{i}(j_);    %Extract the actual index associated with this #'d of i'd payoff neighbord
                    W0(j,i)= (j==i)*wii0s(i)+wij0s(i,j)*(any(j==Np{i}));
                end
                
                
            end
            

            
            avext = externality_gen_2019(nk,nP,Np,beta,W0,A,mmax);

            ext_BS = [ext_BS;fi1/nk]; %Simple type
            ext_B1 = [ext_B1;avext(1)];
            ext_B2 = [ext_B2;avext(2)];
            ext_B3 = [ext_B3;avext(3)];
            ext_B4 = [ext_B4;avext(4)];
            
            
            
        end
        
        
        dx = max(nP);
        da = mean(nP);
   
        cext_A = [cext_A ext_A];
        cext_BS = [cext_BS ext_BS];    
        cext_B1 = [cext_B1 ext_B1];
        cext_B2 = [cext_B2 ext_B2];
        cext_B3 = [cext_B3 ext_B3];
        cext_B4 = [cext_B4 ext_B4];
        
        cdx = [cdx dx];
        cda = [cda da];
        cn = [cn nk];
        
    end
    
    ccext_A = ccext_A+cext_A/MS;
    ccext_BS = ccext_BS+cext_BS/MS;
    ccext_B1 = ccext_B1+cext_B1/MS;
    ccext_B2 = ccext_B2+cext_B2/MS;
    ccext_B3 = ccext_B3+cext_B3/MS;
    ccext_B4 = ccext_B4+cext_B4/MS;

    ccdx = ccdx + cdx/MS;
    ccda = ccda + cda/MS;
    ccn=ccn+cn/MS;
end

diary extval_sim_BA.txt

ccn
ccdx
ccda

ccext_A
ccext_BS
ccext_B1
ccext_B2
ccext_B3
ccext_B4

diary off

timen=datestr(datetime('now','Format','HH:mm:ss.SSS'));
cat1=strcat('BA-n-',num2str(n));
cat2=strcat(cat1,'-');
cat3=strcat(cat2,timen);
save(cat3)


figure;
yl=-2;
yu=2;
LT=2.5;
beta_grid=[-0.9:0.1:0.9]';


subplot(2,2,1);plot(beta_grid,ccext_BS(:,2:4),'k');axis([-1 1 yl yu]);xlabel('\beta');ylabel('ANE')
set(findall(gcf,'-property','FontSize'),'FontSize',12)
set(findall(gca,'-property','LineWidth'),'LineWidth',LT)
title('Simple Type')
subplot(2,2,2);plot(beta_grid,ccext_B1(:,2:4),'k');axis([-1 1 yl yu]);xlabel('\beta');ylabel('ANE')
set(findall(gcf,'-property','FontSize'),'FontSize',12)
set(findall(gca,'-property','LineWidth'),'LineWidth',LT)
title('1st Order')

subplot(2,2,3);plot(beta_grid,ccext_B4(:,2:4),'k');axis([-1 1 yl yu]);xlabel('\beta');ylabel('ANE')
set(findall(gcf,'-property','FontSize'),'FontSize',12)
set(findall(gca,'-property','LineWidth'),'LineWidth',LT)
title('4th Order')

subplot(2,2,4);plot(beta_grid,ccext_A(:,2:4),'k');axis([-1 1 yl yu]);xlabel('\beta');ylabel('ANE')
title('Equilibrium')
% colormap(gray)

set(findall(gcf,'-property','FontSize'),'FontSize',12)
set(findall(gca,'-property','LineWidth'),'LineWidth',LT)